#include "lexer.h"
#include "helpers.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

Lexer lexer_create(const TokenRule* rules, size_t num_rules) {
    return (Lexer){
        .text = NULL,
        .offset = 0,
        .rules = rules,
        .num_rules = num_rules,
    };
}

void lexer_feed(Lexer* lexer, const char* text) {
    assert(lexer != NULL);
    lexer->text = text;
}

bool lexer_next(Lexer* lexer, Token* token) {
    assert(lexer != NULL);
    assert(lexer->text != NULL);

    if(lexer->offset >= strlen(lexer->text)) {
        return false;
    }

    regmatch_t match;
    for(int i = 0; i < lexer->num_rules; i++) {
        TokenRule rule = lexer->rules[i];

        if(regex_search(&match, &rule.regex, lexer->text + lexer->offset)) {
            int length = match.rm_eo;

            char* slice = malloc(length);
            strncpy(slice, lexer->text + lexer->offset, length);
            slice[length] = '\0';

            lexer->offset += length;

            token->type = rule.token_type;
            token->value = slice;
            return true;
        }
    }

    fprintf(stderr, "Unrecognized character: %c\n", *(lexer->text + lexer->offset));
    exit(EXIT_FAILURE);
}
